import * as THREE from 'three'
import { Editor } from './editor';

import { DRACOLoader  } from 'three/examples/jsm/loaders/DRACOLoader';
import { GLTFLoader } from 'three/examples/jsm/loaders/GLTFLoader';

import { TGALoader } from 'three/examples/jsm/loaders/TGALoader.js';

export class Loader {
    private editor: Editor;

    constructor( editor: Editor ) {
        this.editor = editor;
    }

    public createFilesMap( files: any ) {
        let map: { [ id: string ]: any } = {};
    
        for ( var i = 0; i < files.length; i ++ ) {
            let file = files[ i ];
            map[ file.name ] = file;
        }
        return map;
    }
    
    public getFilesFromItemList( items: any, onDone: any ) {
    
        let itemsCount: number = 0;
        let itemsTotal: number = 0;
    
        let files: any = [];
        let filesMap: { [ id: string ]: any } = {};
    
        function onEntryHandled() {
            itemsCount ++;
            if ( itemsCount === itemsTotal ) {
                onDone( files, filesMap );
            }
        }
    
        function handleEntry( entry: any ) {
            if ( entry.isDirectory ) {
                let reader = entry.createReader();
                reader.readEntries( ( entries: any ) => {
                    for ( var i = 0; i < entries.length; i ++ ) {
                        handleEntry( entries[ i ] );
                    }
                    onEntryHandled();
                } );
            } else if ( entry.isFile ) {
                entry.file( ( file: any ) => {
                    files.push( file );
                    filesMap[ entry.fullPath.substr( 1 ) ] = file;
                    onEntryHandled();
                } );
            }
            itemsTotal ++;
        }
    
        for ( var i = 0; i < items.length; i ++ ) {
            var item = items[ i ];
            if ( item.kind === 'file' ) {
                handleEntry( item.webkitGetAsEntry() );
            }
        }
    }

    public loadItemList( items: any ): void {
        this.getFilesFromItemList( items, ( files: any, filesMap: any ) => {
            this.loadFiles( files, filesMap );
        } );
    }

    public loadFiles( files: any, filesMap?: any ): void {
        if ( files.length > 0 ) {
            let filesmap: any = filesMap || this.createFilesMap( files );

            let manager: THREE.LoadingManager = new THREE.LoadingManager();
            manager.setURLModifier( ( url: string ) => {
                url = url.replace(  /^(\.?\/)/, '' );
                let file: any = filesmap[ url ];
                if ( file ) {
                    return URL.createObjectURL( file );
                }
                return url;
            } );

            manager.addHandler( /\.tga$/i, new TGALoader() );
            for ( var i = 0; i < files.length; i ++ ) {
                this.loadFile( files[ i ], manager );
            }
        }
    }

    // this load only glb file in this stripped test
    public loadFile( file: any, manager: THREE.LoadingManager ): void {
        let filename: any = file.name;
        let extension: string = filename.split( '.' ).pop().toLowerCase();
        let reader: FileReader = new FileReader();

        switch ( extension ) {
            case 'glb':
                reader.addEventListener( 'load', ( event: any ) => {
                    let dracoLoader = new DRACOLoader();
                    dracoLoader.setDecoderPath( 'three/examples/js/libs/draco/gltf/' );
                    let loader: GLTFLoader = new GLTFLoader();
                    loader.setDRACOLoader( dracoLoader );
                    loader.parse( event.target.result, '', ( result: any) => {
                        let object = result.scene;
                        object.name = filename;
                        // animations not used in this stripped test
                        this.editor.loadObjectAndAnimation( object );
                    });
                }, false );
                reader.readAsArrayBuffer( file );
                break;
            default:
                console.error( 'Unsupported file format (' + extension + ').' );
                break;
        }
    }
}